# -*- coding: utf-8 -*-
#
from __future__ import print_function

import pipdated

# pylint: disable=wildcard-import
from .time_steppers import *

from .__about__ import (
    __version__,
    __author__,
    __author_email__
    )

if pipdated.needs_checking('parabolic'):
    msg = pipdated.check('parabolic', __version__)
    if msg:
        print(msg)



# -*- coding: utf-8 -*-
#
'''
Routines for one time-stepping of the general equation

.. math::
    \\frac{du}{dt} = F(u).

'''


class ExplicitEuler(object):
    '''
    Explicit Euler method for :math:`u' = F(u)`.
    '''
    order = 1.0

    def __init__(self, problem):
        self.problem = problem
        return

    def step(self, u0, t, dt):
        # (u{k+1} - u{k}) / dt = F(u{k}, t)
        # u{k+1} = u{k} + dt * F(u{k}, t)
        b = self.problem.eval_alpha_M_beta_F(1.0, dt, u0, t)
        u1 = self.problem.solve_alpha_M_beta_F(1.0, 0.0, b, t+dt)
        return u1


class ImplicitEuler(object):
    '''
    Implicit Euler method for :math:`u' = F(u)`.
    '''
    order = 1.0

    def __init__(self, problem):
        self.problem = problem
        return

    def step(self, u0, t, dt):
        # (u{k+1} - u{k}) / dt = F(u{k+1}, t+dt)
        # u{k+1} - dt * F(u{k}, t+dt) = u{k}
        b = self.problem.eval_alpha_M_beta_F(1.0, 0.0, u0, t)
        u1 = self.problem.solve_alpha_M_beta_F(1.0, -dt, b, t+dt)
        return u1


class Trapezoidal(object):
    '''
    Trapezoidal method for :math:`u' = F(u)`. (Known as Crank-Nicolson if
    combined with a second-order discretization in time, or used in an ODE
    context.)
    '''
    order = 2.0

    def __init__(self, problem):
        self.problem = problem
        return

    def step(self, u0, t, dt):
        # (u{k+1} - u{k}) / dt = 1/2 * (F(u{k+1}, t+dt) + F(u{k}, t))
        # u{k+1} - dt/2 * F(u{k+1}, t+dt) = u{k} + dt/2 * F(u{k}, t)
        b = self.problem.eval_alpha_M_beta_F(1.0, 0.5*dt, u0, t)
        u1 = self.problem.solve_alpha_M_beta_F(1.0, -0.5*dt, b, t+dt)
        return u1



# -*- coding: utf-8 -*-
#

__version__ = '0.1.1'
__author__ = u'Nico Schlömer'
__author_email__ = 'nico.schloemer@gmail.com'
__status__ = 'Development Status :: 3 - Alpha'
__license__ = 'License :: OSI Approved :: MIT License'



# -*- coding: utf-8 -*-
#
from setuptools import setup, find_packages
import os
import codecs

# https://packaging.python.org/single_source_version/
base_dir = os.path.abspath(os.path.dirname(__file__))
about = {}
with open(os.path.join(base_dir, 'parabolic', '__about__.py')) as f:
    exec(f.read(), about)


def read(fname):
    try:
        content = codecs.open(
            os.path.join(os.path.dirname(__file__), fname),
            encoding='utf-8'
            ).read()
    except Exception:
        content = ''
    return content


setup(
    name='parabolic',
    version=about['__version__'],
    author=about['__author__'],
    author_email=about['__author_email__'],
    packages=find_packages(),
    description=(
       'Time-steppers for parabolic PDEs'
       ),
    long_description=read('README.rst'),
    url='https://github.com/nschloe/parabolic',
    download_url='https://github.com/nschloe/parabolic/releases',
    license=about['__license__'],
    platforms='any',
    install_requires=[
        'matplotlib',
        'numpy',
        'pipdated',
        ],
    classifiers=[
        about['__status__'],
        about['__license__'],
        'Intended Audience :: Science/Research',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Topic :: Scientific/Engineering :: Mathematics'
        ],
    )
